<?php

namespace Ep\Images;

/**
 * Class Image
 *
 * @package Ep\Images
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/images
 */
class Image
{
    /** @var string */
    protected $data;
    /** @var string */
    protected $path;
    /** @var string */
    protected $realPath;

    protected $magicBytes = [
        'image/jpeg' => "\xFF\xD8\xFF",
        'image/png' => "\x89\x50\x4E\x47\x0D\x0A",
    ];

    /**
     * @param string $data
     */
    public function loadData(string $data): void
    {
        $this->data = $data;
    }

    /**
     * @param string $path
     */
    public function loadFile(string $path): void
    {
        $this->data = file_get_contents($path);
    }

    /**
     * @return string
     */
    public function getData(): string
    {
        return $this->data;
    }

    /**
     * @param string $data
     */
    public function setData(string $data): void
    {
        $this->loadData($data);
        file_put_contents($this->path, $this->data);
    }

    /**
     * @return string
     */
    public function getPath(): string
    {
        return $this->path;
    }

    /**
     * @param string $path
     */
    public function setPath(string $path): void
    {
        $this->path = $path;
    }


    /**
     * @return string
     */
    public function getType(): string
    {
        foreach ($this->magicBytes as $type => $magicByte) {
            if (strpos($this->data, $magicByte) === 0) {
                return $type;
            }
        }

        throw new Exception\InvalidImageTypeException('Ne mogu prepoznati vrstu datoteke.');
    }

    public function resize($width, $height, $cropped = false)
    {
        try {

            $imagick = new \Imagick($this->getPath());

            if ($cropped) {
                $w = $imagick->getImageWidth();
                $h = $imagick->getImageHeight();

                if ($w > $h) {
                    $resize_w = $w * $width / $h;
                    $resize_h = $height;
                } else {
                    $resize_w = $width;
                    $resize_h = $h * $width / $w;
                }

                $imagick->resizeImage($resize_w, $resize_h, \Imagick::FILTER_UNDEFINED, 1, true);
                $imagick->cropImage($width, $height, ($resize_w - $width) / 2, ($resize_h - $height) / 2);
            } else {
                if ($width === 0 || $height === 0) {
                    $ratio = $imagick->getImageWidth() / $imagick->getImageHeight();

                    if ($width === 0) {
                        $width = intval(round($ratio * $width));
                    } else {
                        $height = intval(round($width / $ratio));
                    }
                }

                $imagick->resizeImage($width, $height, \Imagick::FILTER_UNDEFINED, 1, true);
            }

            $resizedPath = sprintf('%s/%s/%s/%s',
                dirname($this->getPath()),
                $cropped ? 'cropped' : '',
                implode('x', [ $imagick->getImageWidth(), $imagick->getImageHeight() ]), // $size = implode('x', [ $width, $height ])
                basename($this->getPath())
            );
            $resizedDir = dirname($resizedPath);


            if (!is_dir($resizedDir)) {
                mkdir($resizedDir, 0755, true);
            }

            $imagick->writeImage($resizedPath);
            $this->loadFile($resizedPath);
            $this->setPath($resizedPath);
        } catch (\ImagickException $e) {
        }
    }
}