<?php

namespace Ep\Images;

use Symfony\Component\Filesystem\Exception\FileNotFoundException;

/**
 * Class ImageManager
 *
 * @package Ep\Images
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/images
 */
class ImageManager
{
    /** @var string */
    protected $basePath;
    protected $resizedDir = 'resized';
    protected $croppedDir = 'cropped';
    /** @var array */
    public $spriteConfig;

    /**
     * ImageManager constructor.
     *
     * @param string $basePath
     * @param array $spriteConfig
     */
    public function __construct(string $basePath, array $spriteConfig = [])
    {
        $this->basePath = rtrim($basePath, '/');
        $this->spriteConfig = $spriteConfig;
    }

    /**
     * @param Image $image
     */
    public function saveImage(Image $image)
    {
        $path = $this->getImagePath($image);
        $dir = dirname($path);

        if (!is_dir($dir)) {
            mkdir($dir, 0777, true);
        }

        file_put_contents($path, $image->getData());
    }

    public function getImage(string $path): Image
    {
        $path = $this->basePath . '/' . ltrim($path, '/');

        if (!file_exists($path)) {
            throw new FileNotFoundException(null, 0, null, $path);
        }


        $image = new Image();
        $image->setPath($path);
        $image->loadFile($path);

        return $image;
    }

    /**
     * @return string
     */
    public function getBasePath()
    {
        return $this->basePath;
    }

    /**
     * @param Image $image
     *
     * @return string
     */
    protected function getImagePath(Image $image): string
    {
        return $this->basePath . '/' . ltrim($image->getPath(), '/');
    }
}